@@ -24,10 +24,17 @@ except ImportError:  # pragma: NO COVER
 
 import grpc
 from grpc.experimental import aio
+from collections.abc import Iterable
+from google.protobuf import json_format
+import json
 import math
 import pytest
 from proto.marshal.rules.dates import DurationRule, TimestampRule
 from proto.marshal.rules import wrappers
+from requests import Response
+from requests import Request, PreparedRequest
+from requests.sessions import Session
+from google.protobuf import json_format
 
 from google.api_core import client_options
 from google.api_core import exceptions as core_exceptions
@@ -43,6 +50,7 @@ from google.cloud.asset_v1p5beta1.services.asset_service import pagers
 from google.cloud.asset_v1p5beta1.services.asset_service import transports
 from google.cloud.asset_v1p5beta1.types import asset_service
 from google.cloud.asset_v1p5beta1.types import assets
+from google.longrunning import operations_pb2
 from google.oauth2 import service_account
 from google.protobuf import timestamp_pb2  # type: ignore
 import google.auth
@@ -94,6 +102,7 @@ def test__get_default_mtls_endpoint():
     [
         (AssetServiceClient, "grpc"),
         (AssetServiceAsyncClient, "grpc_asyncio"),
+        (AssetServiceClient, "rest"),
     ],
 )
 def test_asset_service_client_from_service_account_info(client_class, transport_name):
@@ -107,7 +116,11 @@ def test_asset_service_client_from_service_account_info(client_class, transport_
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("cloudasset.googleapis.com:443")
+        assert client.transport._host == (
+            "cloudasset.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://cloudasset.googleapis.com"
+        )
 
 
 @pytest.mark.parametrize(
@@ -115,6 +128,7 @@ def test_asset_service_client_from_service_account_info(client_class, transport_
     [
         (transports.AssetServiceGrpcTransport, "grpc"),
         (transports.AssetServiceGrpcAsyncIOTransport, "grpc_asyncio"),
+        (transports.AssetServiceRestTransport, "rest"),
     ],
 )
 def test_asset_service_client_service_account_always_use_jwt(
@@ -140,6 +154,7 @@ def test_asset_service_client_service_account_always_use_jwt(
     [
         (AssetServiceClient, "grpc"),
         (AssetServiceAsyncClient, "grpc_asyncio"),
+        (AssetServiceClient, "rest"),
     ],
 )
 def test_asset_service_client_from_service_account_file(client_class, transport_name):
@@ -160,13 +175,18 @@ def test_asset_service_client_from_service_account_file(client_class, transport_
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("cloudasset.googleapis.com:443")
+        assert client.transport._host == (
+            "cloudasset.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://cloudasset.googleapis.com"
+        )
 
 
 def test_asset_service_client_get_transport_class():
     transport = AssetServiceClient.get_transport_class()
     available_transports = [
         transports.AssetServiceGrpcTransport,
+        transports.AssetServiceRestTransport,
     ]
     assert transport in available_transports
 
@@ -183,6 +203,7 @@ def test_asset_service_client_get_transport_class():
             transports.AssetServiceGrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (AssetServiceClient, transports.AssetServiceRestTransport, "rest"),
     ],
 )
 @mock.patch.object(
@@ -326,6 +347,8 @@ def test_asset_service_client_client_options(
             "grpc_asyncio",
             "false",
         ),
+        (AssetServiceClient, transports.AssetServiceRestTransport, "rest", "true"),
+        (AssetServiceClient, transports.AssetServiceRestTransport, "rest", "false"),
     ],
 )
 @mock.patch.object(
@@ -519,6 +542,7 @@ def test_asset_service_client_get_mtls_endpoint_and_cert_source(client_class):
             transports.AssetServiceGrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (AssetServiceClient, transports.AssetServiceRestTransport, "rest"),
     ],
 )
 def test_asset_service_client_client_options_scopes(
@@ -559,6 +583,7 @@ def test_asset_service_client_client_options_scopes(
             "grpc_asyncio",
             grpc_helpers_async,
         ),
+        (AssetServiceClient, transports.AssetServiceRestTransport, "rest", None),
     ],
 )
 def test_asset_service_client_client_options_credentials_file(
@@ -1089,6 +1114,346 @@ async def test_list_assets_async_pages():
             assert page_.raw_page.next_page_token == token
 
 
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        asset_service.ListAssetsRequest,
+        dict,
+    ],
+)
+def test_list_assets_rest(request_type):
+    client = AssetServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"parent": "sample1/sample2"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = asset_service.ListAssetsResponse(
+            next_page_token="next_page_token_value",
+        )
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = asset_service.ListAssetsResponse.pb(return_value)
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.list_assets(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, pagers.ListAssetsPager)
+    assert response.next_page_token == "next_page_token_value"
+
+
+def test_list_assets_rest_required_fields(request_type=asset_service.ListAssetsRequest):
+    transport_class = transports.AssetServiceRestTransport
+
+    request_init = {}
+    request_init["parent"] = ""
+    request = request_type(**request_init)
+    pb_request = request_type.pb(request)
+    jsonified_request = json.loads(
+        json_format.MessageToJson(
+            pb_request,
+            including_default_value_fields=False,
+            use_integers_for_enums=False,
+        )
+    )
+
+    # verify fields with default values are dropped
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).list_assets._get_unset_required_fields(jsonified_request)
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with default values are now present
+
+    jsonified_request["parent"] = "parent_value"
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).list_assets._get_unset_required_fields(jsonified_request)
+    # Check that path parameters and body parameters are not mixing in.
+    assert not set(unset_fields) - set(
+        (
+            "asset_types",
+            "content_type",
+            "page_size",
+            "page_token",
+            "read_time",
+        )
+    )
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with non-default values are left alone
+    assert "parent" in jsonified_request
+    assert jsonified_request["parent"] == "parent_value"
+
+    client = AssetServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+    request = request_type(**request_init)
+
+    # Designate an appropriate value for the returned response.
+    return_value = asset_service.ListAssetsResponse()
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(Session, "request") as req:
+        # We need to mock transcode() because providing default values
+        # for required fields will fail the real version if the http_options
+        # expect actual values for those fields.
+        with mock.patch.object(path_template, "transcode") as transcode:
+            # A uri without fields and an empty body will force all the
+            # request fields to show up in the query_params.
+            pb_request = request_type.pb(request)
+            transcode_result = {
+                "uri": "v1/sample_method",
+                "method": "get",
+                "query_params": pb_request,
+            }
+            transcode.return_value = transcode_result
+
+            response_value = Response()
+            response_value.status_code = 200
+
+            pb_return_value = asset_service.ListAssetsResponse.pb(return_value)
+            json_return_value = json_format.MessageToJson(pb_return_value)
+
+            response_value._content = json_return_value.encode("UTF-8")
+            req.return_value = response_value
+
+            response = client.list_assets(request)
+
+            expected_params = [("$alt", "json;enum-encoding=int")]
+            actual_params = req.call_args.kwargs["params"]
+            assert expected_params == actual_params
+
+
+def test_list_assets_rest_unset_required_fields():
+    transport = transports.AssetServiceRestTransport(
+        credentials=ga_credentials.AnonymousCredentials
+    )
+
+    unset_fields = transport.list_assets._get_unset_required_fields({})
+    assert set(unset_fields) == (
+        set(
+            (
+                "assetTypes",
+                "contentType",
+                "pageSize",
+                "pageToken",
+                "readTime",
+            )
+        )
+        & set(("parent",))
+    )
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_list_assets_rest_interceptors(null_interceptor):
+    transport = transports.AssetServiceRestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.AssetServiceRestInterceptor(),
+    )
+    client = AssetServiceClient(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.AssetServiceRestInterceptor, "post_list_assets"
+    ) as post, mock.patch.object(
+        transports.AssetServiceRestInterceptor, "pre_list_assets"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = asset_service.ListAssetsRequest.pb(
+            asset_service.ListAssetsRequest()
+        )
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = asset_service.ListAssetsResponse.to_json(
+            asset_service.ListAssetsResponse()
+        )
+
+        request = asset_service.ListAssetsRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = asset_service.ListAssetsResponse()
+
+        client.list_assets(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_list_assets_rest_bad_request(
+    transport: str = "rest", request_type=asset_service.ListAssetsRequest
+):
+    client = AssetServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {"parent": "sample1/sample2"}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.list_assets(request)
+
+
+def test_list_assets_rest_flattened():
+    client = AssetServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = asset_service.ListAssetsResponse()
+
+        # get arguments that satisfy an http rule for this method
+        sample_request = {"parent": "sample1/sample2"}
+
+        # get truthy value for each flattened field
+        mock_args = dict(
+            parent="parent_value",
+        )
+        mock_args.update(sample_request)
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = asset_service.ListAssetsResponse.pb(return_value)
+        json_return_value = json_format.MessageToJson(pb_return_value)
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+
+        client.list_assets(**mock_args)
+
+        # Establish that the underlying call was made with the expected
+        # request object values.
+        assert len(req.mock_calls) == 1
+        _, args, _ = req.mock_calls[0]
+        assert path_template.validate(
+            "%s/v1p5beta1/{parent=*/*}/assets" % client.transport._host, args[1]
+        )
+
+
+def test_list_assets_rest_flattened_error(transport: str = "rest"):
+    client = AssetServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # Attempting to call a method with both a request object and flattened
+    # fields is an error.
+    with pytest.raises(ValueError):
+        client.list_assets(
+            asset_service.ListAssetsRequest(),
+            parent="parent_value",
+        )
+
+
+def test_list_assets_rest_pager(transport: str = "rest"):
+    client = AssetServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(Session, "request") as req:
+        # TODO(kbandes): remove this mock unless there's a good reason for it.
+        # with mock.patch.object(path_template, 'transcode') as transcode:
+        # Set the response as a series of pages
+        response = (
+            asset_service.ListAssetsResponse(
+                assets=[
+                    assets.Asset(),
+                    assets.Asset(),
+                    assets.Asset(),
+                ],
+                next_page_token="abc",
+            ),
+            asset_service.ListAssetsResponse(
+                assets=[],
+                next_page_token="def",
+            ),
+            asset_service.ListAssetsResponse(
+                assets=[
+                    assets.Asset(),
+                ],
+                next_page_token="ghi",
+            ),
+            asset_service.ListAssetsResponse(
+                assets=[
+                    assets.Asset(),
+                    assets.Asset(),
+                ],
+            ),
+        )
+        # Two responses for two calls
+        response = response + response
+
+        # Wrap the values into proper Response objs
+        response = tuple(asset_service.ListAssetsResponse.to_json(x) for x in response)
+        return_values = tuple(Response() for i in response)
+        for return_val, response_val in zip(return_values, response):
+            return_val._content = response_val.encode("UTF-8")
+            return_val.status_code = 200
+        req.side_effect = return_values
+
+        sample_request = {"parent": "sample1/sample2"}
+
+        pager = client.list_assets(request=sample_request)
+
+        results = list(pager)
+        assert len(results) == 6
+        assert all(isinstance(i, assets.Asset) for i in results)
+
+        pages = list(client.list_assets(request=sample_request).pages)
+        for page_, token in zip(pages, ["abc", "def", "ghi", ""]):
+            assert page_.raw_page.next_page_token == token
+
+
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.AssetServiceGrpcTransport(
@@ -1170,6 +1535,7 @@ def test_transport_get_channel():
     [
         transports.AssetServiceGrpcTransport,
         transports.AssetServiceGrpcAsyncIOTransport,
+        transports.AssetServiceRestTransport,
     ],
 )
 def test_transport_adc(transport_class):
@@ -1184,6 +1550,7 @@ def test_transport_adc(transport_class):
     "transport_name",
     [
         "grpc",
+        "rest",
     ],
 )
 def test_transport_kind(transport_name):
@@ -1311,6 +1678,7 @@ def test_asset_service_transport_auth_adc(transport_class):
     [
         transports.AssetServiceGrpcTransport,
         transports.AssetServiceGrpcAsyncIOTransport,
+        transports.AssetServiceRestTransport,
     ],
 )
 def test_asset_service_transport_auth_gdch_credentials(transport_class):
@@ -1405,11 +1773,23 @@ def test_asset_service_grpc_transport_client_cert_source_for_mtls(transport_clas
             )
 
 
+def test_asset_service_http_transport_client_cert_source_for_mtls():
+    cred = ga_credentials.AnonymousCredentials()
+    with mock.patch(
+        "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
+    ) as mock_configure_mtls_channel:
+        transports.AssetServiceRestTransport(
+            credentials=cred, client_cert_source_for_mtls=client_cert_source_callback
+        )
+        mock_configure_mtls_channel.assert_called_once_with(client_cert_source_callback)
+
+
 @pytest.mark.parametrize(
     "transport_name",
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_asset_service_host_no_port(transport_name):
@@ -1420,7 +1800,11 @@ def test_asset_service_host_no_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("cloudasset.googleapis.com:443")
+    assert client.transport._host == (
+        "cloudasset.googleapis.com:443"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://cloudasset.googleapis.com"
+    )
 
 
 @pytest.mark.parametrize(
@@ -1428,6 +1812,7 @@ def test_asset_service_host_no_port(transport_name):
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_asset_service_host_with_port(transport_name):
@@ -1438,7 +1823,33 @@ def test_asset_service_host_with_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("cloudasset.googleapis.com:8000")
+    assert client.transport._host == (
+        "cloudasset.googleapis.com:8000"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://cloudasset.googleapis.com:8000"
+    )
+
+
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "rest",
+    ],
+)
+def test_asset_service_client_transport_session_collision(transport_name):
+    creds1 = ga_credentials.AnonymousCredentials()
+    creds2 = ga_credentials.AnonymousCredentials()
+    client1 = AssetServiceClient(
+        credentials=creds1,
+        transport=transport_name,
+    )
+    client2 = AssetServiceClient(
+        credentials=creds2,
+        transport=transport_name,
+    )
+    session1 = client1.transport.list_assets._session
+    session2 = client2.transport.list_assets._session
+    assert session1 != session2
 
 
 def test_asset_service_grpc_transport_channel():
@@ -1775,6 +2186,7 @@ async def test_transport_close_async():
 
 def test_transport_close():
     transports = {
+        "rest": "_session",
         "grpc": "_grpc_channel",
     }
 
@@ -1792,6 +2204,7 @@ def test_transport_close():
 
 def test_client_ctx():
     transports = [
+        "rest",
         "grpc",
     ]
     for transport in transports:
